@@ -1,22 +1,40 @@
 from os import path
 import requests
 import time
-from .HTMLparsers import getSchiHubPDF
+from .HTMLparsers import getSchiHubPDF, SciHubUrls
 import random
+from urllib.parse import urljoin
+from .NetInfo import NetInfo
+
+def setSciHubUrl():
+    r = requests.get(NetInfo.SciHub_URLs_repo, headers=NetInfo.HEADERS)
+    links = SciHubUrls(r.text)
+    found = False
+
+    for l in links:
+        r = requests.get(l, headers=NetInfo.HEADERS)
+        if r.status_code == 200:
+            found = True
+            NetInfo.SciHub_URL = l
+            break
+
+    if found:
+        print("\nUsing {} as Sci-Hub instance".format(NetInfo.SciHub_URL))
+    else:
+        print("\nNo working Sci-Hub instance found!\nIf in your country Sci-Hub is not available consider using a VPN")
+        NetInfo.SciHub_URL = "https://sci-hub.st/"
 
 
-HEADERS = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36'}
-SciHub_URL = "https://sci-hub.st/"
+def getSaveDir(folder, fname):
+    dir_ = path.join(folder, fname)
+    n = 1
+    while path.exists(dir_):
+       n += 1
+       dir_ = path.join(folder, "("+str(n)+")"+fname)
+    
+    return dir_
 
-def saveFile(file_name,content, paper,dwn_source):    
-    if path.exists(file_name):
-       file_name_temp = file_name
-       n = 2
-       while path.exists(file_name_temp):
-           file_name_temp = "("+str(n)+")"+file_name 
-           n += 1
-       file_name = file_name_temp
-         
+def saveFile(file_name,content, paper,dwn_source): 
     f = open(file_name, 'wb')
     f.write(content)
     f.close()
@@ -26,8 +44,8 @@ def saveFile(file_name,content, paper,dwn_source):
             
     
 def downloadPapers(papers, dwnl_dir, num_limit):
-    global HEADERS
-    global SciHub_URL
+    if NetInfo.SciHub_URL==None:
+        setSciHubUrl()
     
     num_downloaded = 0
     paper_number = 1
@@ -37,7 +55,7 @@ def downloadPapers(papers, dwnl_dir, num_limit):
             print("Download {} of {} -> {}".format(paper_number, len(papers), p.title))
             paper_number += 1
             
-            pdf_dir = dwnl_dir + p.getFileName()
+            pdf_dir = getSaveDir(dwnl_dir, p.getFileName())
                                     
             faild = 0
             while p.downloaded==False and faild!=4:        
@@ -45,20 +63,19 @@ def downloadPapers(papers, dwnl_dir, num_limit):
                     
                     dwn_source = 1 #1 scihub 2 scholar 
                     if faild==0 and p.DOI!=None:
-                        url = SciHub_URL + p.DOI
+                        url = urljoin(NetInfo.SciHub_URL, p.DOI)
                     if faild==1 and p.scholar_link!=None:
-                        url = SciHub_URL + p.scholar_link
+                        url = urljoin(NetInfo.SciHub_URL, p.scholar_link)
                         
                     if faild==2 and p.scholar_link!=None and p.scholar_link[-3:]=="pdf":
                         url = p.scholar_link
                         dwn_source = 2
                     if faild==3 and p.pdf_link!=None:
                         url = p.pdf_link
-                        dwn_source = 2
-                        
+                        dwn_source = 2                        
                 
                     if url!="":
-                        r = requests.get(url, headers=HEADERS)
+                        r = requests.get(url, headers=NetInfo.HEADERS)
                         content_type = r.headers.get('content-type')
                         
                         if dwn_source==1 and 'application/pdf' not in content_type:
@@ -66,7 +83,7 @@ def downloadPapers(papers, dwnl_dir, num_limit):
                             
                             pdf_link = getSchiHubPDF(r.text)
                             if(pdf_link != None):
-                                r = requests.get(pdf_link, headers=HEADERS)
+                                r = requests.get(pdf_link, headers=NetInfo.HEADERS)
                                 content_type = r.headers.get('content-type')
     
                         if 'application/pdf' in content_type:
